
/**
 * Number box class for dragging targets for "number drag game"
 * reference solution
 * 
 * @author Jim Teresco 
 */

import objectdraw.*;
import java.awt.*;

public class NumberBox
{


    // a border, divider, a few Text objects needed, plus int
    // values to track goal, total so far
    private FramedRect outline;
    private Text goalLabel, soFarLabel;
    private int goal, soFar;
    
    public NumberBox(double x, double y, int goal, double boxSize, DrawingCanvas c) {
    
        // frame
        outline = new FramedRect(x, y, boxSize, boxSize, c);
        // no need to remember the divider line
        new Line(x, y+boxSize/2, x+boxSize, y+boxSize/2, c);
        
        this.goal = goal;
        soFar = 0; 
        
        // labels for goal and amount so far
        soFarLabel = new Text("0", 0, 0, c);
        soFarLabel.setFontSize((int)boxSize/3);
        soFarLabel.setBold(true);
        soFarLabel.move(x + boxSize/2 - soFarLabel.getWidth()/2,
                        y + boxSize/4 - soFarLabel.getHeight()/2);
        soFarLabel.setColor(Color.green);
        goalLabel = new Text(""+goal, 0, 0, c);
        goalLabel.setFontSize((int)boxSize/3);
        goalLabel.setBold(true);
        goalLabel.move(x + boxSize/2 - goalLabel.getWidth()/2,
                       y + 3*boxSize/4 - goalLabel.getHeight()/2);
        goalLabel.setColor(Color.blue);

    }
  
    // check containment of a point
    public boolean contains(Location point) {
        
        return outline.contains(point);
    }
    
    // see if this box has its total already accumulated
    public boolean matched() {
        
        return goal == soFar;
    }
    
    // add in a number just dropped into this bx
    public void dropIn(int amount) {
        
        if (soFar > goal) {
            soFar -= amount;
        }
        else {
            soFar += amount;
        }
        soFarLabel.setText("" + soFar);
        soFarLabel.moveTo(outline.getX() + outline.getWidth()/2 - soFarLabel.getWidth()/2,
                        outline.getY() + outline.getHeight()/4 - soFarLabel.getHeight()/2);
        if (soFar == goal) {
            soFarLabel.setColor(Color.blue);
        }
        else if (soFar < goal) {
            soFarLabel.setColor(Color.green);
        }
        else {
            soFarLabel.setColor(Color.red);
        }
    }
}
