import objectdraw.*;
import java.awt.*;
import java.util.Random;
import java.util.ArrayList;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;

/*
 * Starter code for "number drag game" lab problem
 *
 * Jim Teresco, The College of Saint Rose, CSC 252, Spring 2016
 *
 */

public class NumberDragGame extends WindowController implements ActionListener {
    
    // information about, and the collection of targets
    private static final int NUM_TARGETS = 5;
    private static final int MIN_GOAL = 25;
    private static final int MAX_GOAL = 60;
    private ArrayList<NumberBox> targets;

    // the range of numbers to be generated and dragged
    private static final int MAX_NUMBER = 20;
    
    // the newest number on the canvas
    private Text newestNumber;
    
    // JLabels and counter for timer and number of numbers used
    private JLabel timerLabel;
    private JLabel numbersUsedLabel;
    private int numbersUsed = 0;
    
    // and our start button
    private JButton startButton;
    
    // our timer
    private CountUpTimer timer;
    
    // the number generator
    private NumberGenerator generator;
    
    // remember target size for use in number generation
    private double targetSize;
    
    // we'll have some random numbers here, so just this one Random
    private Random r = new Random();
    
    // track if the game is over
    private boolean gameOver = false;
    
    // dragging support
    private Text draggingText;
    private Location lastMouse;
    
    public void begin() {
        
        // GUI setup
        Container contentPane = getContentPane();
        
        // But now we'll create a JPanel, which we will then use
        // to group together our JButton and JLabels.
        JPanel southPanel = new JPanel();
        
        // labels
        timerLabel = new JLabel("Elapsed time: 0.0");
        southPanel.add(timerLabel);
        
        numbersUsedLabel = new JLabel("Numbers used: 0");
        southPanel.add(numbersUsedLabel);
        
        // start button
        startButton = new JButton("Start");
        southPanel.add(startButton);
        startButton.addActionListener(this);
        
        contentPane.add(southPanel, BorderLayout.SOUTH);
        contentPane.validate();
                
        targets = new ArrayList<NumberBox>();
        
        // place the targets evenly spaced across the bottom of
        // the canvas
        targetSize = canvas.getWidth()/(2*NUM_TARGETS+1);
        for (int i = 0; i < NUM_TARGETS; i++) {
            NumberBox nb = new NumberBox(targetSize + 2*i*targetSize, 
                                         canvas.getHeight() - 1.5*targetSize,
                                         r.nextInt(MAX_GOAL-MIN_GOAL) + MIN_GOAL, 
                                         targetSize, canvas);
            targets.add(nb);
        }
        
        newestNumber = null;
    }
    
    public void actionPerformed(ActionEvent e) {
        
        startButton.setEnabled(false);
        // start the timer
        timer = new CountUpTimer(timerLabel);
        // launch the number generator
        generator = new NumberGenerator(this);
        
    }
    
    // method to be called periodically to add a new number to the collection
    // on the canvas
    public void nextNumber() {
        
        // choose the number: 1 to MAX_NUMBER
        int value = r.nextInt(MAX_NUMBER)+1;
        
        // and a random location for it
        double xLoc = targetSize + r.nextDouble()*(canvas.getWidth() - 2*targetSize);
        double yLoc = targetSize + r.nextDouble()*(canvas.getHeight() - 3*targetSize);
        
        newestNumber = new Text("" + value, xLoc, yLoc, canvas);
        
    }
    
    public void onMousePress(Location point) {
    
        // if the game is still running, see if we picked up a number to
        // start dragging
        if (gameOver) return;
        
        draggingText = null;
        if (newestNumber != null && newestNumber.contains(point)) {
           draggingText = newestNumber;
           lastMouse = point;
        }
    }
    
    public void onMouseRelease(Location point) {
    
       // only do anything here if we are dragging
       if (draggingText != null) {
            draggingText.move(point.getX() - lastMouse.getX(),
                              point.getY() - lastMouse.getY());

            // see if we dropped in a target, and check for game
            // over along the way -- gameOver will be set back to
            // false if any target is not completed
            gameOver = true;
            for (NumberBox box : targets) {
                if (!box.matched()) {
                    if (box.contains(point)) {
                        int value = Integer.parseInt(draggingText.getText());
                        box.dropIn(value);
                        draggingText.removeFromCanvas();
                        numbersUsed++;
                        numbersUsedLabel.setText("Numbers used: " + numbersUsed);
                    }
                }
                // see if box matched now
                if (!box.matched()) {
                    gameOver = false;
                }
            }
            if (gameOver) {
                timer.stopTimer();
                generator.stopGenerating();
            }
        }
    }
  
    public void onMouseDrag(Location point) {
    
        // standard dragging procedure
        if (draggingText != null) {
            draggingText.move(point.getX() - lastMouse.getX(),
                              point.getY() - lastMouse.getY());
            lastMouse = point;
        }
    }
    
}
